import os
import logging
import requests
from typing import Dict, Optional, List, Any, Union
import base64
import mimetypes
import json

logger = logging.getLogger(__name__)


class OllamaConnector:
    def __init__(self):
        self.url = os.getenv("OLLAMA_URL", "http://localhost:11434")
        self.model = os.getenv("OLLAMA_MODEL", "llama2")
        self._check_vision_support()
        logger.info(
            f"Initialized OllamaConnector with URL: {self.url} and model: {self.model}"
        )

    def _check_vision_support(self) -> None:
        """Check if the current model supports vision capabilities."""
        try:
            response = requests.get(f"{self.url}/api/tags")
            if response.status_code == 200:
                models = response.json().get("models", [])
                for model in models:
                    if model["name"] == self.model:
                        self.supports_vision = model.get("details", {}).get(
                            "vision", False
                        )
                        logger.info(
                            f"Model {self.model} vision support: {self.supports_vision}"
                        )
                        return
            self.supports_vision = False
            logger.warning(f"Could not determine vision support for model {self.model}")
        except Exception as e:
            self.supports_vision = False
            logger.error(f"Error checking vision support: {str(e)}")

    def _is_supported_text_file(self, mime_type: str) -> bool:
        """Check if the file type is supported for text processing."""
        supported_types = [
            "text/plain",
            "application/pdf",
            "text/html",
            "application/json",
            "text/markdown",
            "text/csv",
        ]
        return mime_type in supported_types

    def _process_text_file(self, file_path: str) -> str:
        """Process a text file and return its content."""
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                return f.read()
        except Exception as e:
            logger.error(f"Error processing text file: {str(e)}")
            raise

    def _process_pdf_file(self, file_path: str) -> str:
        """Process a PDF file and return its text content."""
        try:
            import PyPDF2

            text = ""
            with open(file_path, "rb") as f:
                pdf_reader = PyPDF2.PdfReader(f)
                for page in pdf_reader.pages:
                    text += page.extract_text() + "\n"
            return text
        except Exception as e:
            logger.error(f"Error processing PDF file: {str(e)}")
            raise

    def process_file(self, file_path: str, mime_type: str) -> str:
        """Process a file and return its content as text."""
        if not self._is_supported_text_file(mime_type):
            raise ValueError(f"Unsupported file type: {mime_type}")

        if mime_type == "application/pdf":
            return self._process_pdf_file(file_path)
        else:
            return self._process_text_file(file_path)

    def process_image(self, image_path: str) -> str:
        """Process an image and return its base64 encoded content."""
        if not self.supports_vision:
            raise ValueError("Current model does not support vision")

        try:
            with open(image_path, "rb") as f:
                return base64.b64encode(f.read()).decode("utf-8")
        except Exception as e:
            logger.error(f"Error processing image: {str(e)}")
            raise

    def generate_response(
        self,
        prompt: str,
        context: Optional[Union[str, List[Dict[str, str]]]] = None,
        image_data: Optional[str] = None,
    ) -> str:
        """
        Generate a response from the Ollama model.

        Args:
            prompt (str): The user's input prompt
            context (Optional[Union[str, List[Dict[str, str]]]]): Additional context for the model
            image_data (Optional[str]): Base64 encoded image data

        Returns:
            str: The model's response
        """
        try:
            url = f"{self.url}/api/generate"

            # Format the prompt with context if provided
            if context:
                if isinstance(context, list):
                    # If it's a list of messages, format it as a conversation
                    context_str = "\n".join(
                        [f"{msg['role']}: {msg['content']}" for msg in context]
                    )
                    full_prompt = f"{context_str}\n\n{prompt}"
                else:
                    # If it's a string (like file content), prepend it to the prompt
                    full_prompt = f"Context:\n{context}\n\nQuestion: {prompt}"
            else:
                full_prompt = prompt

            payload = {
                "model": self.model,
                "prompt": full_prompt,
                "stream": False,
            }

            if image_data and self.supports_vision:
                payload["images"] = [image_data]

            logger.debug(
                f"Sending request to Ollama with payload: {json.dumps(payload, indent=2)}"
            )
            response = requests.post(url, json=payload)
            response.raise_for_status()

            # Remove "assistant: " prefix if present
            response_text = response.json().get("response", "")
            if response_text.startswith("assistant: "):
                response_text = response_text[10:].lstrip()

            return response_text
        except requests.exceptions.RequestException as e:
            error_msg = f"Error communicating with Ollama: {str(e)}"
            if hasattr(e.response, "text"):
                error_msg += f"\nResponse: {e.response.text}"
            logger.error(error_msg, exc_info=True)
            return "I apologize, but I'm having trouble connecting to the AI service right now. Please try again later."
        except Exception as e:
            logger.error(f"Unexpected error: {str(e)}")
            raise
