import os
import logging
import tempfile
from telegram import Update
from telegram.ext import (
    Application,
    CommandHandler,
    MessageHandler,
    filters,
    ContextTypes,
)
from typing import Optional, Dict, Tuple, List

from src.connectors.ollama_connector import OllamaConnector
from src.memory.memory_manager import MemoryManager

logger = logging.getLogger(__name__)


class TelegramBot:
    def __init__(self):
        self.token = os.getenv("TELEGRAM_BOT_TOKEN")
        if not self.token:
            raise ValueError("TELEGRAM_BOT_TOKEN not found in environment variables")

        logger.info("Initializing Telegram bot components...")
        self.ollama = OllamaConnector()
        self.memory = MemoryManager()
        self.application = Application.builder().token(self.token).build()

        # Store current conversation for each user
        self.user_conversations: Dict[int, int] = {}

        # Register handlers
        logger.info("Registering command handlers...")
        self._register_handlers()

    def _register_handlers(self) -> None:
        """Register all command and message handlers."""
        handlers = [
            CommandHandler("start", self.start_command),
            CommandHandler("help", self.help_command),
            CommandHandler("conv", self.conv_command),
            CommandHandler("renameconv", self.renameconv_command),
            CommandHandler("reset", self.reset_command),
            CommandHandler("sysprompt", self.sysprompt_command),
            MessageHandler(filters.TEXT & ~filters.COMMAND, self.handle_message),
            MessageHandler(filters.Document.ALL, self.handle_document),
            MessageHandler(filters.PHOTO, self.handle_photo),
        ]
        for handler in handlers:
            self.application.add_handler(handler)

    def _get_user_conv(self, user_id: int) -> int:
        """Get the current conversation ID for a user."""
        if user_id not in self.user_conversations:
            self.user_conversations[user_id] = 0
        return self.user_conversations[user_id]

    def _parse_conv_id(
        self, args: Tuple[Optional[int], str]
    ) -> Tuple[Optional[int], str]:
        """Parse conversation ID from arguments."""
        if not args:
            return None, "Please provide a conversation number."
        try:
            return int(args[0]), ""
        except ValueError:
            return None, "Please provide a valid conversation number."

    async def start_command(
        self, update: Update, context: ContextTypes.DEFAULT_TYPE
    ) -> None:
        """Handle the /start command."""
        user = update.effective_user
        logger.info(f"New user started conversation: {user.id} ({user.username})")
        self.user_conversations[user.id] = 0
        await update.message.reply_text(
            "Hello! I'm your AI assistant powered by Ollama. "
            "Use /help to see available commands."
        )

    async def help_command(
        self, update: Update, context: ContextTypes.DEFAULT_TYPE
    ) -> None:
        """Handle the /help command."""
        help_text = """Available commands:

/conversation management:
/conv [number] - Switch to conversation
/conv - List all conversations
/renameconv [number] [title] - Rename a conversation
/reset [number] - Reset a conversation

/system prompt:
/sysprompt [number] [prompt] - Set system prompt
/sysprompt [number] - View system prompt

/help - Show this help message"""
        await update.message.reply_text(help_text)

    async def conv_command(
        self, update: Update, context: ContextTypes.DEFAULT_TYPE
    ) -> None:
        """Handle the /conv command."""
        user_id = update.effective_user.id
        args = context.args

        if not args:
            # List all conversations
            conversations = self.memory.list_conversations(user_id)
            if not conversations:
                await update.message.reply_text("You have no conversations yet.")
                return

            response = "Your conversations:\n"
            for conv_id, conv_data in conversations.items():
                title_str = f" - {conv_data['title']}" if conv_data["title"] else ""
                sysprompt_str = (
                    f" (has system prompt)" if conv_data["system_prompt"] else ""
                )
                response += f"{conv_id}{title_str}{sysprompt_str}\n"
            await update.message.reply_text(response)
            return

        conv_id, error_msg = self._parse_conv_id(args)
        if error_msg:
            await update.message.reply_text(error_msg)
            return

        self.user_conversations[user_id] = conv_id
        conversations = self.memory.list_conversations(user_id)
        conv_data = conversations.get(conv_id, {"title": None, "system_prompt": None})
        title = conv_data["title"] or "unnamed"
        sysprompt_str = " (has system prompt)" if conv_data["system_prompt"] else ""
        await update.message.reply_text(
            f"Switched to conversation {conv_id} ({title}){sysprompt_str}"
        )

    async def renameconv_command(
        self, update: Update, context: ContextTypes.DEFAULT_TYPE
    ) -> None:
        """Handle the /renameconv command."""
        user_id = update.effective_user.id
        args = context.args

        if len(args) < 2:
            await update.message.reply_text(
                "Please provide a conversation number and title.\n"
                "Example: /renameconv 0 My First Chat"
            )
            return

        conv_id, error_msg = self._parse_conv_id(args)
        if error_msg:
            await update.message.reply_text(error_msg)
            return

        title = " ".join(args[1:])
        if self.memory.rename_conversation(user_id, conv_id, title):
            await update.message.reply_text(
                f"Conversation {conv_id} renamed to '{title}'"
            )
        else:
            await update.message.reply_text(f"Conversation {conv_id} does not exist.")

    async def reset_command(
        self, update: Update, context: ContextTypes.DEFAULT_TYPE
    ) -> None:
        """Handle the /reset command."""
        user_id = update.effective_user.id
        args = context.args

        if not args:
            await update.message.reply_text(
                "Please provide a conversation number to reset.\n" "Example: /reset 0"
            )
            return

        conv_id, error_msg = self._parse_conv_id(args)
        if error_msg:
            await update.message.reply_text(error_msg)
            return

        if self.memory.reset_conversation(user_id, conv_id):
            await update.message.reply_text(f"Conversation {conv_id} has been reset.")
        else:
            await update.message.reply_text(f"Conversation {conv_id} does not exist.")

    async def sysprompt_command(
        self, update: Update, context: ContextTypes.DEFAULT_TYPE
    ) -> None:
        """Handle the /sysprompt command."""
        user_id = update.effective_user.id
        args = context.args

        if not args:
            await update.message.reply_text(
                "Please provide a conversation number.\n"
                'Example: /sysprompt 0 "You are a helpful assistant"'
            )
            return

        conv_id, error_msg = self._parse_conv_id(args)
        if error_msg:
            await update.message.reply_text(error_msg)
            return

        if len(args) == 1:
            # View system prompt
            prompt = self.memory.get_system_prompt(user_id, conv_id)
            if prompt is None:
                await update.message.reply_text(
                    f"Conversation {conv_id} has no system prompt set."
                )
            else:
                await update.message.reply_text(
                    f"System prompt for conversation {conv_id}:\n\n{prompt}"
                )
        else:
            # Set system prompt
            prompt = " ".join(args[1:])
            if self.memory.set_system_prompt(user_id, conv_id, prompt):
                await update.message.reply_text(
                    f"System prompt set for conversation {conv_id}."
                )
            else:
                await update.message.reply_text(
                    f"Conversation {conv_id} does not exist."
                )

    async def _send_thinking_message(
        self, update: Update, context: ContextTypes.DEFAULT_TYPE
    ) -> int:
        """Send a thinking message and return its message_id."""
        thinking_message = await update.message.reply_text("🤔 Thinking...")
        return thinking_message.message_id

    async def _delete_message(
        self, chat_id: int, message_id: int, context: ContextTypes.DEFAULT_TYPE
    ) -> None:
        """Delete a message by its ID."""
        try:
            await context.bot.delete_message(chat_id=chat_id, message_id=message_id)
        except Exception as e:
            logger.error(f"Error deleting message: {str(e)}")

    def _split_message(self, text: str, max_length: int = 4000) -> List[str]:
        """Split a long message into chunks that fit within Telegram's message limit."""
        if len(text) <= max_length:
            return [text]

        # Split by paragraphs first
        paragraphs = text.split("\n\n")
        chunks = []
        current_chunk = []
        current_length = 0

        for paragraph in paragraphs:
            # If adding this paragraph would exceed the limit, start a new chunk
            if current_length + len(paragraph) + 2 > max_length and current_chunk:
                chunks.append("\n\n".join(current_chunk))
                current_chunk = []
                current_length = 0

            current_chunk.append(paragraph)
            current_length += len(paragraph) + 2  # +2 for the \n\n we'll add back

        # Add the last chunk if it's not empty
        if current_chunk:
            chunks.append("\n\n".join(current_chunk))

        return chunks

    async def _send_long_message(
        self, chat_id: int, text: str, context: ContextTypes.DEFAULT_TYPE
    ) -> None:
        """Send a message that might be too long for Telegram by splitting it into chunks."""
        chunks = self._split_message(text)
        for i, chunk in enumerate(chunks):
            if i == 0:
                await context.bot.send_message(chat_id=chat_id, text=chunk)
            else:
                # Add a continuation indicator for subsequent chunks
                await context.bot.send_message(
                    chat_id=chat_id, text=f"...(continued)\n\n{chunk}"
                )

    async def handle_message(
        self, update: Update, context: ContextTypes.DEFAULT_TYPE
    ) -> None:
        """Handle incoming text messages."""
        user_id = update.effective_user.id
        user_message = update.message.text
        conv_id = self._get_user_conv(user_id)

        # Send thinking message
        thinking_message_id = await self._send_thinking_message(update, context)

        try:
            # Get conversation context
            conversation_context = self.memory.get_conversation_context(
                user_id, conv_id
            )

            # Generate response
            response = self.ollama.generate_response(
                prompt=user_message, context=conversation_context
            )

            # Delete thinking message
            await self._delete_message(
                update.effective_chat.id, thinking_message_id, context
            )

            # Send response (handling long messages)
            await self._send_long_message(update.effective_chat.id, response, context)

            # Save to memory
            self.memory.add_message(user_id, conv_id, "user", user_message)
            self.memory.add_message(user_id, conv_id, "assistant", response)

        except Exception as e:
            logger.error(f"Error processing message: {str(e)}")
            # Delete thinking message
            await self._delete_message(
                update.effective_chat.id, thinking_message_id, context
            )
            await update.message.reply_text(
                "Sorry, I encountered an error while processing your message. Please try again."
            )

    async def handle_document(
        self, update: Update, context: ContextTypes.DEFAULT_TYPE
    ) -> None:
        """Handle incoming documents."""
        user_id = update.effective_user.id
        document = update.message.document
        conv_id = self._get_user_conv(user_id)

        # Send thinking message
        thinking_message_id = await self._send_thinking_message(update, context)

        try:
            # Check if file type is supported
            mime_type = document.mime_type
            if not self.ollama._is_supported_text_file(mime_type):
                await self._delete_message(
                    update.effective_chat.id, thinking_message_id, context
                )
                await update.message.reply_text(
                    "Sorry, I can only process text-based files (PDF, TXT, HTML, etc.)."
                )
                return

            # Download the file
            with tempfile.NamedTemporaryFile(delete=False) as temp_file:
                file = await context.bot.get_file(document.file_id)
                await file.download_to_drive(temp_file.name)

                # Process the file
                file_content = self.ollama.process_file(temp_file.name, mime_type)

                # Generate response using file content as context
                response = self.ollama.generate_response(
                    prompt=update.message.caption or "Please analyze this document.",
                    context=file_content,
                )

                # Delete thinking message
                await self._delete_message(
                    update.effective_chat.id, thinking_message_id, context
                )

                # Send response (handling long messages)
                await self._send_long_message(
                    update.effective_chat.id, response, context
                )

        except Exception as e:
            logger.error(f"Error processing document: {str(e)}")
            # Delete thinking message
            await self._delete_message(
                update.effective_chat.id, thinking_message_id, context
            )
            await update.message.reply_text(
                "Sorry, I encountered an error while processing your document. Please try again."
            )
        finally:
            # Clean up temporary file
            if "temp_file" in locals():
                os.unlink(temp_file.name)

    async def handle_photo(
        self, update: Update, context: ContextTypes.DEFAULT_TYPE
    ) -> None:
        """Handle incoming photos."""
        if not self.ollama.supports_vision:
            await update.message.reply_text(
                "Sorry, the current model does not support image analysis."
            )
            return

        user_id = update.effective_user.id
        photo = update.message.photo[-1]  # Get the highest quality photo
        conv_id = self._get_user_conv(user_id)

        # Send thinking message
        thinking_message_id = await self._send_thinking_message(update, context)

        try:
            # Download the photo
            with tempfile.NamedTemporaryFile(delete=False, suffix=".jpg") as temp_file:
                file = await context.bot.get_file(photo.file_id)
                await file.download_to_drive(temp_file.name)

                # Process the image
                image_data = self.ollama.process_image(temp_file.name)

                # Generate response using image
                response = self.ollama.generate_response(
                    prompt=update.message.caption or "Please describe this image.",
                    image_data=image_data,
                )

                # Delete thinking message
                await self._delete_message(
                    update.effective_chat.id, thinking_message_id, context
                )

                # Send response (handling long messages)
                await self._send_long_message(
                    update.effective_chat.id, response, context
                )

        except Exception as e:
            logger.error(f"Error processing photo: {str(e)}")
            # Delete thinking message
            await self._delete_message(
                update.effective_chat.id, thinking_message_id, context
            )
            await update.message.reply_text(
                "Sorry, I encountered an error while processing your image. Please try again."
            )
        finally:
            # Clean up temporary file
            if "temp_file" in locals():
                os.unlink(temp_file.name)

    def run(self) -> None:
        """Start the bot."""
        logger.info("Starting bot polling...")
        self.application.run_polling()
