import os
import json
import logging
from typing import List, Dict, Optional
from datetime import datetime
from dotenv import load_dotenv

logger = logging.getLogger(__name__)


class MemoryManager:
    def __init__(self):
        # Load environment variables
        load_dotenv()

        self.memory_dir = os.getenv("MEMORY_DIR", "./memory")
        self.default_system_prompt = os.getenv(
            "DEFAULT_SYSTEM_PROMPT", "You are a helpful AI assistant."
        )
        print(self.default_system_prompt)
        os.makedirs(self.memory_dir, exist_ok=True)
        logger.info(f"Initialized MemoryManager with directory: {self.memory_dir}")

    def _get_user_file(self, user_id: int) -> str:
        """Get the path to a user's memory file."""
        return os.path.join(self.memory_dir, f"user_{user_id}.json")

    def get_user_memory(self, user_id: int) -> Dict:
        """Retrieve a user's conversation history."""
        file_path = self._get_user_file(user_id)
        logger.debug(f"Retrieving memory for user {user_id} from {file_path}")
        if os.path.exists(file_path):
            try:
                with open(file_path, "r") as f:
                    return json.load(f)
            except json.JSONDecodeError as e:
                logger.error(f"Error reading memory file for user {user_id}: {str(e)}")
                return {"conversations": {}}
        return {"conversations": {}}

    def save_user_memory(self, user_id: int, memory: Dict) -> None:
        """Save a user's conversation history."""
        file_path = self._get_user_file(user_id)
        logger.debug(f"Saving memory for user {user_id} to {file_path}")
        try:
            with open(file_path, "w") as f:
                json.dump(memory, f, indent=2)
        except Exception as e:
            logger.error(f"Error saving memory file for user {user_id}: {str(e)}")

    def add_message(self, user_id: int, conv_id: int, role: str, content: str) -> None:
        """Add a new message to the user's conversation history."""
        logger.debug(
            f"Adding message for user {user_id} in conversation {conv_id} (role: {role})"
        )
        memory = self.get_user_memory(user_id)

        # Initialize conversation if it doesn't exist
        if str(conv_id) not in memory["conversations"]:
            memory["conversations"][str(conv_id)] = {
                "messages": [],
                "title": None,
                "system_prompt": self.default_system_prompt,
            }

        message = {
            "role": role,
            "content": content,
            "timestamp": datetime.now().isoformat(),
        }
        memory["conversations"][str(conv_id)]["messages"].append(message)
        self.save_user_memory(user_id, memory)

    def get_conversation_context(
        self, user_id: int, conv_id: int, max_messages: int = 5
    ) -> str:
        """Get the recent conversation context for a user."""
        logger.debug(
            f"Getting conversation context for user {user_id} in conversation {conv_id}"
        )
        memory = self.get_user_memory(user_id)

        if str(conv_id) not in memory["conversations"]:
            return ""

        conversation = memory["conversations"][str(conv_id)]
        context_parts = []

        # Add system prompt if it exists
        if conversation["system_prompt"]:
            context_parts.append(f"System: {conversation['system_prompt']}")

        # Add conversation messages
        messages = conversation["messages"][-max_messages:]
        context_parts.extend([f"{msg['role']}: {msg['content']}" for msg in messages])

        return "\n".join(context_parts)

    def rename_conversation(self, user_id: int, conv_id: int, title: str) -> bool:
        """Rename a conversation."""
        logger.debug(f"Renaming conversation {conv_id} for user {user_id} to '{title}'")
        memory = self.get_user_memory(user_id)

        if str(conv_id) not in memory["conversations"]:
            return False

        memory["conversations"][str(conv_id)]["title"] = title
        self.save_user_memory(user_id, memory)
        return True

    def reset_conversation(self, user_id: int, conv_id: int) -> bool:
        """Reset a conversation by clearing its messages while preserving the system prompt."""
        memory = self.get_user_memory(user_id)

        if str(conv_id) not in memory["conversations"]:
            return False

        # Get the current system prompt
        current_system_prompt = memory["conversations"][str(conv_id)].get(
            "system_prompt", self.default_system_prompt
        )

        # Reset the conversation while preserving the system prompt
        memory["conversations"][str(conv_id)] = {
            "messages": [],
            "title": memory["conversations"][str(conv_id)].get("title", None),
            "system_prompt": current_system_prompt,
        }

        self.save_user_memory(user_id, memory)
        logger.info(
            f"Reset conversation {conv_id} for user {user_id} while preserving system prompt"
        )
        return True

    def list_conversations(self, user_id: int) -> Dict[int, Dict[str, Optional[str]]]:
        """List all conversations for a user."""
        logger.debug(f"Listing conversations for user {user_id}")
        memory = self.get_user_memory(user_id)
        return {
            int(conv_id): {
                "title": conv_data["title"],
                "system_prompt": conv_data["system_prompt"],
            }
            for conv_id, conv_data in memory["conversations"].items()
        }

    def set_system_prompt(
        self, user_id: int, conv_id: int, prompt: Optional[str]
    ) -> bool:
        """Set the system prompt for a conversation."""
        logger.debug(
            f"Setting system prompt for conversation {conv_id} of user {user_id}"
        )
        memory = self.get_user_memory(user_id)

        if str(conv_id) not in memory["conversations"]:
            return False

        memory["conversations"][str(conv_id)]["system_prompt"] = prompt
        self.save_user_memory(user_id, memory)
        return True

    def get_system_prompt(self, user_id: int, conv_id: int) -> Optional[str]:
        """Get the system prompt for a conversation."""
        logger.debug(
            f"Getting system prompt for conversation {conv_id} of user {user_id}"
        )
        memory = self.get_user_memory(user_id)

        if str(conv_id) not in memory["conversations"]:
            return None

        return memory["conversations"][str(conv_id)]["system_prompt"]
